@@ -27,19 +27,17 @@
 
 from __future__ import absolute_import
 
+import itertools
 import six
 import random
 from functools import wraps, partial
 
-from twisted.python.failure import Failure
-from twisted.internet.error import ReactorNotRunning
-
 import txaio
 
 from autobahn.util import ObservableMixin
 from autobahn.websocket.util import parse_url
 from autobahn.wamp.types import ComponentConfig, SubscribeOptions, RegisterOptions
-from autobahn.wamp.exception import SessionNotReady
+from autobahn.wamp.exception import SessionNotReady, ApplicationError
 from autobahn.wamp.auth import create_authenticator
 
 
@@ -94,12 +92,12 @@ def _validate_endpoint(endpoint, check_native_endpoint=None):
             for k in ['path']:
                 if k not in endpoint:
                     raise ValueError(
-                        "'{}' required for 'tcp' endpoint config".format(k)
+                        "'{}' required for 'unix' endpoint config".format(k)
                     )
             for k in ['host', 'port', 'tls']:
                 if k in endpoint:
                     raise ValueError(
-                        "'{}' not valid for in 'tcp' endpoint config".format(k)
+                        "'{}' not valid in 'unix' endpoint config".format(k)
                     )
         else:
             assert False, 'should not arrive here'
@@ -226,7 +224,7 @@ class _Transport(object):
     """
 
     def __init__(self, idx, kind, url, endpoint, serializers,
-                 max_retries=15,
+                 max_retries=-1,
                  max_retry_delay=300,
                  initial_retry_delay=1.5,
                  retry_delay_growth=1.5,
@@ -279,13 +277,15 @@ class _Transport(object):
     def can_reconnect(self):
         if self._permanent_failure:
             return False
-        return self.connect_attempts < self.max_retries
+        if self.max_retries == -1:
+            return True
+        return self.connect_attempts < self.max_retries + 1
 
     def next_delay(self):
         if self.connect_attempts == 0:
             # if we never tried before, try immediately
             return 0
-        elif self.connect_attempts >= self.max_retries:
+        elif self.max_retries != -1 and self.connect_attempts >= self.max_retries + 1:
             raise RuntimeError('max reconnects reached')
         else:
             self.retry_delay = self.retry_delay * self.retry_delay_growth
@@ -362,7 +362,8 @@ class Component(ObservableMixin):
             self.on('join', do_registration)
         return decorator
 
-    def __init__(self, main=None, transports=None, config=None, realm=u'default', extra=None, authentication=None):
+    def __init__(self, main=None, transports=None, config=None, realm=u'default', extra=None,
+                 authentication=None, session_factory=None, is_fatal=None):
         """
         :param main: After a transport has been connected and a session
             has been established and joined to a realm, this (async)
@@ -379,7 +380,7 @@ class Component(ObservableMixin):
             containing the following configuration keys:
 
                 - ``type`` (optional): ``websocket`` (default) or ``rawsocket``
-                - ``url``: the WAMP URL
+                - ``url``: the router URL
                 - ``endpoint`` (optional, derived from URL if not provided):
                     - ``type``: "tcp" or "unix"
                     - ``host``, ``port``: only for TCP
@@ -393,14 +394,23 @@ class Component(ObservableMixin):
 
         :type transports: None or unicode or list of dicts
 
-        :param config: Session configuration (currently unused?)
-        :type config: None or dict
-
         :param realm: the realm to join
         :type realm: unicode
 
         :param authentication: configuration of authenticators
         :type authentication: dict mapping auth_type to dict
+
+        :param session_factory: if None, ``ApplicationSession`` is
+            used, otherwise a callable taking a single ``config`` argument
+            that is used to create a new `ApplicationSession` instance.
+        :type session_factory: callable
+
+        :param is_fatal: a callable taking a single argument, an
+            ``Exception`` instance. The callable should return ``True`` if
+            this error is "fatal", meaning we should not try connecting to
+            the current transport again. The default behavior (on None) is
+            to always return ``False``
+        :type is_fatal: callable taking one arg, or None
         """
         self.set_valid_events(
             [
@@ -410,12 +420,16 @@ class Component(ObservableMixin):
                 'ready',        # fired by ApplicationSession
                 'leave',        # fired by ApplicationSession
                 'disconnect',   # fired by ApplicationSession
+                'connectfailure',   # fired by base class
             ]
         )
 
-        if main is not None and not callable(main):
-            raise RuntimeError('"main" must be a callable if given')
+        if is_fatal is not None and not callable(is_fatal):
+            raise ValueError('"is_fatal" must be a callable or None')
+        self._is_fatal = is_fatal
 
+        if main is not None and not callable(main):
+            raise ValueError('"main" must be a callable if given')
         self._entry = main
 
         # use WAMP-over-WebSocket to localhost when no transport is specified at all
@@ -450,6 +464,8 @@ class Component(ObservableMixin):
         # XXX should have some checkconfig support
         self._authentication = authentication or {}
 
+        if session_factory:
+            self.session_factory = session_factory
         self._realm = realm
         self._extra = extra
 
@@ -460,11 +476,145 @@ class Component(ObservableMixin):
                 return True
         return False
 
-    def start(self, reactor_or_loop):
-        raise RuntimeError('not implemented')
+    def _start(self, loop=None):
+        """
+        This starts the Component, which means it will start connecting
+        (and re-connecting) to its configured transports. A Component
+        runs until it is "done", which means one of:
+
+        - There was a "main" function defined, and it completed successfully;
+        - Something called ``.leave()`` on our session, and we left successfully;
+        - ``.stop()`` was called, and completed successfully;
+        - none of our transports were able to connect successfully (failure);
+
+        :returns: a Future/Deferred which will resolve (to ``None``) when we are
+            "done" or with an error if something went wrong.
+        """
+
+        # this future will be returned, and thus has the semantics
+        # specified in the docstring.
+        done_f = txaio.create_future()
+
+        # Create a generator of transports that .can_reconnect()
+        transport_gen = itertools.cycle(self._transports)
+
+        # this is a 1-element list so we can set it from closures in
+        # this function
+        transport_candidate = [0]
+
+        def error(fail):
+            self.log.info("Internal error {msg}", msg=txaio.failure_message(fail))
+            self.log.debug("{tb}", tb=txaio.failure_format_traceback(fail))
+            txaio.reject(done_f, fail)
+
+        def attempt_connect(_):
+
+            def handle_connect_error(fail):
+                # FIXME - make txaio friendly
+                # Can connect_f ever be in a cancelled state?
+                # if txaio.using_asyncio and isinstance(fail.value, asyncio.CancelledError):
+                #     unrecoverable_error = True
+
+                self.log.debug(u'component failed: {error}', error=txaio.failure_message(fail))
+                self.log.debug(u'{tb}', tb=txaio.failure_format_traceback(fail))
+                # If this is a "fatal error" that will never work,
+                # we bail out now
+                if isinstance(fail.value, ApplicationError):
+                    self.log.error(u"{msg}", msg=fail.value.error_message())
+
+                elif isinstance(fail.value, OSError):
+                    # failed to connect entirely, like nobody
+                    # listening etc.
+                    self.log.info(u"Connection failed: {msg}", msg=txaio.failure_message(fail))
+
+                elif self._is_ssl_error(fail.value):
+                    # Quoting pyOpenSSL docs: "Whenever
+                    # [SSL.Error] is raised directly, it has a
+                    # list of error messages from the OpenSSL
+                    # error queue, where each item is a tuple
+                    # (lib, function, reason). Here lib, function
+                    # and reason are all strings, describing where
+                    # and what the problem is. See err(3) for more
+                    # information."
+                    self.log.error(u"TLS failure: {reason}", reason=fail.value.args[1])
+                else:
+                    self.log.error(
+                        u'Connection failed: {error}',
+                        error=txaio.failure_message(fail),
+                    )
+
+                if self._is_fatal is None:
+                    is_fatal = False
+                else:
+                    is_fatal = self._is_fatal(fail.value)
+                if is_fatal:
+                    self.log.info("Error was fatal; failing transport")
+                    transport_candidate[0].failed()
+
+                txaio.call_later(0, transport_check, None)
+                return
+
+            def notify_connect_error(fail):
+                chain_f = txaio.create_future()
+                # hmm, if connectfailure took a _Transport instead of
+                # (or in addition to?) self it could .failed() the
+                # transport and we could do away with the is_fatal
+                # listener?
+                handler_f = self.fire('connectfailure', self, fail.value)
+                txaio.add_callbacks(
+                    handler_f,
+                    lambda _: txaio.reject(chain_f, fail),
+                    lambda _: txaio.reject(chain_f, fail)
+                )
+                return chain_f
+
+            def connect_error(fail):
+                notify_f = notify_connect_error(fail)
+                txaio.add_callbacks(notify_f, None, handle_connect_error)
+
+            def session_done(x):
+                txaio.resolve(done_f, None)
+
+            connect_f = self._connect_once(loop, transport_candidate[0])
+            txaio.add_callbacks(connect_f, session_done, connect_error)
+
+        def transport_check(_):
+            self.log.debug('Entering re-connect loop')
+
+            if not self._can_reconnect():
+                err_msg = u"Component failed: Exhausted all transport connect attempts"
+                self.log.info(err_msg)
+                try:
+                    raise RuntimeError(err_msg)
+                except RuntimeError as e:
+                    txaio.reject(done_f, e)
+                    return
+
+            while True:
+
+                transport = next(transport_gen)
+
+                if transport.can_reconnect():
+                    transport_candidate[0] = transport
+                    break
+
+            delay = transport.next_delay()
+            self.log.debug(
+                'trying transport {transport_idx} using connect delay {transport_delay}',
+                transport_idx=transport.idx,
+                transport_delay=delay,
+            )
+
+            delay_f = txaio.sleep(delay)
+            txaio.add_callbacks(delay_f, attempt_connect, error)
+
+        # issue our first event, then start the reconnect loop
+        start_f = self.fire('start', loop, self)
+        txaio.add_callbacks(start_f, transport_check, error)
+        return done_f
 
     def stop(self):
-        raise RuntimeError('not implemented')
+        return self._session.leave()
 
     def _connect_once(self, reactor, transport):
 
@@ -481,7 +631,7 @@ class Component(ObservableMixin):
         def create_session():
             cfg = ComponentConfig(self._realm, self._extra)
             try:
-                session = self.session_factory(cfg)
+                self._session = session = self.session_factory(cfg)
                 for auth_name, auth_config in self._authentication.items():
                     authenticator = create_authenticator(auth_name, **auth_config)
                     session.add_authenticator(authenticator)
@@ -508,8 +658,14 @@ class Component(ObservableMixin):
                         "session leaving '{details.reason}'",
                         details=details,
                     )
-                    if self._entry and not txaio.is_called(done):
-                        txaio.resolve(done, None)
+                    if not txaio.is_called(done):
+                        if details.reason in [u"wamp.close.normal"]:
+                            txaio.resolve(done, None)
+                        else:
+                            f = txaio.create_failure(
+                                ApplicationError(details.reason)
+                            )
+                            txaio.reject(done, f)
                 session.on('leave', on_leave)
 
                 # if we were given a "main" procedure, we run through
@@ -553,15 +709,17 @@ class Component(ObservableMixin):
                             self.log.warn(
                                 u"Session disconnected uncleanly"
                             )
-                        # eg the session has left the realm, and the transport was properly
-                        # shut down. successfully finish the connection
-                        txaio.resolve(done, None)
+                        else:
+                            # eg the session has left the realm, and the transport was properly
+                            # shut down. successfully finish the connection
+                            txaio.resolve(done, None)
                 session.on('disconnect', on_disconnect)
 
                 # return the fresh session object
                 return session
 
         transport.connect_attempts += 1
+
         d = self._connect_transport(reactor, transport, create_session)
 
         def on_connect_sucess(proto):
@@ -628,7 +786,7 @@ class Component(ObservableMixin):
         self.on('ready', fn)
 
 
-def _run(reactor, components):
+def _run(reactor, components, done_callback):
     """
     Internal helper. Use "run" method from autobahn.twisted.wamp or
     autobahn.asyncio.wamp
@@ -695,11 +853,7 @@ def _run(reactor, components):
 
     def all_done(arg):
         log.debug("All components ended; stopping reactor")
-        if isinstance(arg, Failure):
-            log.error("Something went wrong: {log_failure}", failure=arg)
-        try:
-            reactor.stop()
-        except ReactorNotRunning:
-            pass
+        done_callback(reactor, arg)
+
     txaio.add_callbacks(done_d, all_done, all_done)
     return done_d
