@@ -425,10 +425,15 @@ class SubscribeOptions(object):
         'details',
         'details_arg',
         'get_retained',
-        'correlation',
+        'correlation_id',
+        'correlation_uri',
+        'correlation_is_anchor',
+        'correlation_is_last',
     )
 
-    def __init__(self, match=None, details=None, details_arg=None, get_retained=None, correlation=None):
+    def __init__(self, match=None, details=None, details_arg=None, get_retained=None,
+                 correlation_id=None, correlation_uri=None, correlation_is_anchor=None,
+                 correlation_is_last=None):
         """
 
         :param match: The topic matching method to be used for the subscription.
@@ -461,7 +466,10 @@ class SubscribeOptions(object):
 
         self.get_retained = get_retained
 
-        self.correlation = correlation
+        self.correlation_id = correlation_id
+        self.correlation_uri = correlation_uri
+        self.correlation_is_anchor = correlation_is_anchor
+        self.correlation_is_last = correlation_is_last
 
     def message_attr(self):
         """
@@ -570,7 +578,10 @@ class PublishOptions(object):
         'eligible_authid',
         'eligible_authrole',
         'retain',
-        'correlation',
+        'correlation_id',
+        'correlation_uri',
+        'correlation_is_anchor',
+        'correlation_is_last',
     )
 
     def __init__(self,
@@ -583,7 +594,10 @@ class PublishOptions(object):
                  eligible_authid=None,
                  eligible_authrole=None,
                  retain=None,
-                 correlation=None):
+                 correlation_id=None,
+                 correlation_uri=None,
+                 correlation_is_anchor=None,
+                 correlation_is_last=None):
         """
 
         :param acknowledge: If ``True``, acknowledge the publication with a success or
@@ -635,7 +649,10 @@ class PublishOptions(object):
         self.eligible_authrole = eligible_authrole
         self.retain = retain
 
-        self.correlation = correlation
+        self.correlation_id = correlation_id
+        self.correlation_uri = correlation_uri
+        self.correlation_is_anchor = correlation_is_anchor
+        self.correlation_is_last = correlation_is_last
 
     def message_attr(self):
         """
@@ -689,16 +706,34 @@ class RegisterOptions(object):
         'concurrency',
         'force_reregister',
         'details_arg',
-        'correlation',
+        'correlation_id',
+        'correlation_uri',
+        'correlation_is_anchor',
+        'correlation_is_last',
     )
 
-    def __init__(self, match=None, invoke=None, concurrency=None, details_arg=None,
-                 force_reregister=None, correlation=None):
+    def __init__(self, match=None, invoke=None, concurrency=None, details_arg=None, force_reregister=None,
+                 correlation_id=None, correlation_uri=None, correlation_is_anchor=None,
+                 correlation_is_last=None):
         """
+        :param match: Type of matching to use on the URI (`exact`, `prefix` or `wildcard`)
+
+        :param invoke: Type of invoke mechanism to use (`single`, `first`, `last`, `roundrobin`, `random`)
+
+        :param concurrency: if used, the number of times a particular
+            endpoint may be called concurrently (e.g. if this is 3, and
+            there are already 3 calls in-progress a 4th call will receive
+            an error)
 
         :param details_arg: When invoking the endpoint, provide call details
-           in this keyword argument to the callable.
+            in this keyword argument to the callable.
         :type details_arg: str
+
+        :param force_reregister: if True, any other session that has
+            already registered this URI will be 'kicked out' and this
+            session will become the one that's registered (the previous
+            session must have used `force_reregister=True` as well)
+
         """
         assert(match is None or (type(match) == six.text_type and match in [u'exact', u'prefix', u'wildcard']))
         assert(invoke is None or (type(invoke) == six.text_type and invoke in [u'single', u'first', u'last', u'roundrobin', u'random']))
@@ -712,7 +747,10 @@ class RegisterOptions(object):
         self.details_arg = details_arg
         self.force_reregister = force_reregister
 
-        self.correlation = correlation
+        self.correlation_id = correlation_id
+        self.correlation_uri = correlation_uri
+        self.correlation_is_anchor = correlation_is_anchor
+        self.correlation_is_last = correlation_is_last
 
     def message_attr(self):
         """
@@ -812,13 +850,19 @@ class CallOptions(object):
     __slots__ = (
         'on_progress',
         'timeout',
-        'correlation',
+        'correlation_id',
+        'correlation_uri',
+        'correlation_is_anchor',
+        'correlation_is_last',
     )
 
     def __init__(self,
                  on_progress=None,
                  timeout=None,
-                 correlation=None):
+                 correlation_id=None,
+                 correlation_uri=None,
+                 correlation_is_anchor=None,
+                 correlation_is_last=None):
         """
 
         :param on_progress: A callback that will be called when the remote endpoint
@@ -834,7 +878,10 @@ class CallOptions(object):
         self.on_progress = on_progress
         self.timeout = timeout
 
-        self.correlation = correlation
+        self.correlation_id = correlation_id
+        self.correlation_uri = correlation_uri
+        self.correlation_is_anchor = correlation_is_anchor
+        self.correlation_is_last = correlation_is_last
 
     def message_attr(self):
         """
